#include "DFRobot_Gesture_Touch.h"
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27, 16, 2); // I2C address 0x27, 16 column and 2 rows

#ifdef __AVR__
  #include "SoftwareSerial.h"
  SoftwareSerial    mySerial(10, 11);
#elif defined ESP_PLATFORM
  #include "HardwareSerial.h"
  HardwareSerial    mySerial(1);
#endif

DFRobot_Gesture_Touch   DFGT(&mySerial); // init sensor object, request write and read function

int count = 0;
bool flag = false;
bool clear_lcd = false;

ISR(TIMER1_COMPA_vect) {
  if (flag == true){
    count ++;
    if (count >= 4){
      clear_lcd = true;
      flag = false;
      count = 0;
    }
  }
}
 
void configure_timer1() {
  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1 = 0;
  OCR1A = 31249;            // compare match register 16MHz/256/2Hz-1
  TCCR1B |= (1 << WGM12);   // CTC mode
  TCCR1B |= (1 << CS12);    // 256 prescaler 
}

void init_timer1() {
  TIMSK1 |= (1 << OCIE1A);  // enable timer compare interrupt
}

void USART0_init()
{
  /* seteaza baud rate la 9600 */
  UBRR0 = 8;
 
  /* porneste transmitatorul */
  UCSR0B = (1 << TXEN0) | (1 << RXEN0);
 
  /* seteaza formatul frame-ului: 8 biti de date, 1 bit de stop, fara paritate */
  UCSR0C &= ~(1 << USBS0);
  UCSR0C |= (3 << UCSZ00);
}
 
void USART0_transmit(unsigned char data) {
    /* așteaptă până când buffer-ul e gol */
    while(!(UCSR0A & (1<<UDRE0)));
 
    /* pune datele în buffer; transmisia va porni automat în urma scrierii */
    UDR0 = data;
}
 
char USART0_receive()
{
  /* asteapta cat timp bufferul e gol */
  while (!(UCSR0A & (1 << RXC0)));
 
  /* returneaza datele din buffer */
  return UDR0;
}

void write(const char* text){
    for(unsigned int nr = 0; nr < strlen(text); nr++){
        USART0_transmit(char(text[nr]));
    }
}

void setup()
{
  mySerial.begin(9600);
  delay(1000);
  while(mySerial.available())  // flush serial buffer
    mySerial.read();

  DFGT.enableFunction(DFGT_FUN_ALL); // enable all functions

  lcd.init(); // initialize the lcd
  lcd.backlight();
  lcd.clear();

  lcd.setCursor(0, 0);
  lcd.print("Touchless");
  lcd.setCursor(2, 1);
  lcd.print("Control");

  // deactivate intrerupts
  cli();  
  USART0_init();

  configure_timer1();
  init_timer1();
  // activate intrerupts
  sei();
}

void loop()
{
  int8_t    rslt = DFGT.getAnEvent();  // get an event that data saved in serial buffer
  if (clear_lcd){
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("Waiting");
        lcd.setCursor(2, 1);
        lcd.print("command");
        clear_lcd = false;
        flag = false;
        count = 0;
      }

  if(rslt != DF_ERR) {    
    if (flag) return;
    switch(rslt) {
      case DFGT_EVT_BACK: {
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("Command:");
        lcd.setCursor(2, 1);
        lcd.print("BACK");
        write("3\n");
        flag = true;
      } break;
      case DFGT_EVT_FORWARD: {
          lcd.clear();
          lcd.setCursor(0, 0);
          lcd.print("Command:");
          lcd.setCursor(2, 1);
          lcd.print("FORWARD");
          write("4\n");
          flag = true;
      } break;
      case DFGT_EVT_RIGHT: {
          lcd.clear();
          lcd.setCursor(0, 0);
          lcd.print("Command:");
          lcd.setCursor(2, 1);
          lcd.print("RIGHT");
          write("1\n");
          flag = true;
      } break;
      case DFGT_EVT_LEFT: {
          lcd.clear();
          lcd.setCursor(0, 0);
          lcd.print("Command:");
          lcd.setCursor(2, 1);
          lcd.print("LEFT");
          write("2\n");
          flag = true;
      } break;
      case DFGT_EVT_PULLUP: {
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("Command:");
        lcd.setCursor(2, 1);
        lcd.print("PULL UP");
        write("5\n");
      } break;
      case DFGT_EVT_PULLDOWN: {
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("Command:");
        lcd.setCursor(2, 1);
        lcd.print("PULL DOWN");
        write("6\n");
      } break;
      case DFGT_EVT_PULLREMOVE: {
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("Command:");
        lcd.setCursor(2, 1);
        lcd.print("PULL REMOVE");
        write("7\n");
        flag = true;
      } break;
      case DFGT_EVT_TOUCH1: {
        Serial.println("get event touch1");
      } break;
      case DFGT_EVT_TOUCH2: {
        Serial.println("get event touch2");
      } break;
      case DFGT_EVT_TOUCH3: {
        Serial.println("get event touch3");
      } break;
      case DFGT_EVT_TOUCH4: {
        Serial.println("get event touch4");
      } break;
      case DFGT_EVT_TOUCH5: {
        Serial.println("get event touch5");
      } break;
    }
  }
}